//Water effects modified from splatsh mod by PrincessPlushima
//https://mb.srb2.org/addons/splatsh.2809/

local P_TeleportMove = P_MoveOrigin or P_TeleportMove

//This code relies on splats, so if you don't have splats, there's no point
//Remember when I was developing this in 2.2.8?
if not MF2_SPLAT return end

freeslot(
	"MT_VEPHSPLATSH",
	"S_VEPHSPLATSH",
	"MT_VEPHSPLATSHTAIL",
	"S_VEPHSPLATSHTAIL"
)

addHook("MobjSpawn", function(actor)
	actor.flags2 = $|MF2_SPLAT
	actor.renderflags = RF_SLOPESPLAT|RF_NOSPLATBILLBOARD
	actor.spriteyscale = FRACUNIT/2
	P_CreateFloorSpriteSlope(actor)
	
	local tracer = P_SpawnMobj(actor.x,actor.y,actor.z, MT_VEPHSPLATSHTAIL)
	if tracer and tracer.valid
		tracer.flags2 = $|MF2_SPLAT
		tracer.renderflags = RF_SLOPESPLAT|RF_NOSPLATBILLBOARD
		tracer.spriteyscale = FRACUNIT
		P_CreateFloorSpriteSlope(tracer)
		actor.tracer = tracer
	end
end, MT_VEPHSPLATSH)

addHook("MobjRemoved", function(actor)
	if actor.tracer and actor.tracer.valid then P_RemoveMobj(actor.tracer) end
end, MT_VEPHSPLATSH)

function A_VEPHSPLATSH(actor, var1, var2)
	if not (actor and actor.valid) then return end
	local tracer = actor.tracer
	local mo = actor.target
	local player = mo and mo.valid and mo.player
	
	actor.reactiontime = $-1
	if (actor.reactiontime <= 0) then
		if tracer and tracer.valid
			P_RemoveMobj(tracer)
		end
		P_RemoveMobj(actor)
		return
	end
	
	local scalefactor = ((actor.info.reactiontime-actor.reactiontime)*FRACUNIT)/28
	local deltafactor = 8
	
	if player
		if (player.powers[pw_shield] & SH_NOSTACK) == SH_ELEMENTAL
			actor.color = SKINCOLOR_KETCHUP
			actor.colorized = true
		end
		if mo.eflags & MFE_VERTICALFLIP
			deltafactor = $*-1
		end
		
		if actor.reactiontime >= actor.info.reactiontime - 1
		and mo.skin == "veph" and player.vephswipe
			actor.momx = mo.momx*2/3 + P_ReturnThrustX(actor, actor.angle, actor.scale*2)
			actor.momy = mo.momy*2/3 + P_ReturnThrustY(actor, actor.angle, actor.scale*2)
			P_TryMove(actor, actor.x + mo.momx, actor.y + mo.momy, true)
			
			actor.flags2 = $ | MF2_BOSSNOTRAP
			actor.momz = P_GetMobjGravity(mo)*8
		end
	end
	
	if actor.flags2&MF2_AMBUSH
		scalefactor = $*3/2
		deltafactor = $*3/2
	elseif not (actor.flags2&MF2_BOSSNOTRAP)
		actor.momx = P_ReturnThrustX(actor, actor.angle, actor.scale*8)
		actor.momy = P_ReturnThrustY(actor, actor.angle, actor.scale*8)
	end
	
	if actor.flags2&MF2_SLIDEPUSH
		scalefactor = $*2/3
		deltafactor = $*3/2
	end
	
	actor.frame = ($&~FF_TRANSMASK)|(min(max((actor.info.reactiontime-actor.reactiontime)/(actor.info.reactiontime/9), 1), 9)*FF_TRANS10)
	
	actor.spritexscale = scalefactor
	if actor.floorspriteslope and actor.floorspriteslope.valid
		actor.floorspriteslope.o = {
			x = actor.x,
			y = actor.y,
			z = actor.z
		}
		actor.floorspriteslope.zdelta = (actor.reactiontime*FRACUNIT)/deltafactor
		actor.floorspriteslope.xydirection = actor.angle
	end
	
	if tracer and tracer.valid
		tracer.frame = ($&~FF_TRANSMASK)|(actor.frame&FF_TRANSMASK)
		
		tracer.spritexscale = scalefactor
		if tracer.floorspriteslope and tracer.floorspriteslope.valid
			tracer.floorspriteslope.o = {
				x = actor.x,
				y = actor.y,
				z = actor.z
			}
			tracer.floorspriteslope.xydirection = actor.angle
			tracer.floorspriteslope.zdelta = (actor.reactiontime*FRACUNIT)/-deltafactor
		end
		
		tracer.scale = actor.scale
		P_TeleportMove(tracer,
		actor.x,//+P_ReturnThrustX(actor, actor.angle, actor.scale*8),
		actor.y,//+P_ReturnThrustY(actor, actor.angle, actor.scale*8),
		actor.z)
		tracer.angle = actor.angle
		tracer.color = actor.color
		tracer.colorized = actor.colorized
	end
end

mobjinfo[MT_VEPHSPLATSH] = {
	spawnstate = S_VEPHSPLATSH,
	reactiontime = 16,
	speed = 10*FRACUNIT,
	radius = 1*FRACUNIT,
	height = 1*FRACUNIT,
	damage = 3,
	spawnhealth = 0,
	dispoffset = 0,
	flags = MF_NOCLIPTHING|MF_NOGRAVITY|MF_MISSILE|MF_NOBLOCKMAP|MF_SCENERY
}

states[S_VEPHSPLATSH] = {
	sprite = SPR_VPPT,
	frame = A|TR_TRANS50,
	tics = 1,
	action = A_VEPHSPLATSH,
	nextstate = S_VEPHSPLATSH
}

mobjinfo[MT_VEPHSPLATSHTAIL] = {
	spawnstate = S_VEPHSPLATSHTAIL,
	reactiontime = 45,
	speed = 10*FRACUNIT,
	radius = 1*FRACUNIT,
	height = 1*FRACUNIT,
	damage = 3,
	spawnhealth = 0,
	dispoffset = 0,
	flags = MF_NOCLIPTHING|MF_NOGRAVITY|MF_NOBLOCKMAP|MF_SCENERY|MF_NOTHINK
}

states[S_VEPHSPLATSHTAIL] = {
	sprite = SPR_VPPT,
	frame = B|TR_TRANS50,
	tics = 45,
	action = None,
	nextstate = S_VEPHSPLATSHTAIL
}